import Channel from 'jschannel'
import EventEmitter from 'eventemitter3'

export default class KioskClient {
  constructor (kioskFrame) {
    this.channel = Channel.build({
      window: kioskFrame,
      origin: '*',
      scope: 'kiosk'
    })

    this.emitters = {}

    this.channel.bind('downloadProgress', this.downloadProgress)
    this.channel.bind('downloadCompleted', this.downloadCompleted)
    this.channel.bind('downloadError', this.downloadError)
  }

  downloadProgress = (context, { url, progress }) => {
    const emitter = this.emitters[url]
    emitter.emit('progress', progress)
  }

  downloadCompleted = (context, { url, localUrl }) => {
    const emitter = this.emitters[url]
    emitter.emit('end', localUrl)
    // download is done, delete emitter so that it's recreated for the next request
    this.emitters[url] = null
  }

  downloadError = (context, { url, error }) => {
    const emitter = this.emitters[url]
    emitter.emit('error', error)
    // download errored, delete emitter so that it's recreated for the next request
    this.emitters[url] = null
  }

  cache (url) {
    let emitter = this.emitters[url]

    if (!emitter) {
      emitter = new EventEmitter()
      this.emitters[url] = emitter
    }

    this.channel.notify({ method: 'cache', params: url })

    return emitter
  }

  uncache (url) {
    return new Promise((resolve, reject) => {
      this.channel.call({
        method: 'uncache',
        params: url,
        success: resolve,
        error: reject
      })
    })
  }

  log (message) {
    this.channel.notify({ method: 'log', params: message })
  }

  error (err) {
    err = typeof err === 'string' ? { message: err } : err
    this.channel.notify({ method: 'error', params: [ err.message, err.stack ] })
  }
}

